using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace WebApplication2.FileManagement
{
    public static class FileManagementUtil
    {
        private static readonly char[] InvalidFileNameChars = Path.GetInvalidFileNameChars();
        public static string SafeFileName(string inName)
        {
            if (string.IsNullOrEmpty(inName)) return "";

            char[] buffer = inName.ToCharArray();
            for (int i = 0; i < buffer.Length; i++)
            {
                char c = buffer[i];

                bool invalid = false;
                foreach (char ic in InvalidFileNameChars)
                {
                    if (c == ic)
                    {
                        invalid = true;
                        break;
                    }
                }
                if (invalid)
                {
                    buffer[i] = '_';
                }
                // use 200 bytes as max
                if (i > 200)
                    break;
            }
            return new string(buffer).Trim();
        }

        public static string SafeFileName(string inName, bool shouldStripSpaces)
        {
            if (shouldStripSpaces)
            {
                if (string.IsNullOrEmpty(inName)) return "";
                return SafeFileName(inName).Replace(" ", "");
            }

            return SafeFileName(inName);
        }

        static readonly char[] ArrBadchars = Path.GetInvalidFileNameChars(); //"/\\|:*[]?".ToCharArray();

        public static string SafeFileName2(string inFilename)
        {

            if (string.IsNullOrEmpty(inFilename)) return "";

            StringBuilder sb = new StringBuilder();
            for (int i = 0; i <= inFilename.Length - 1; i++)
            {
                if (inFilename.Substring(i, 1).IndexOfAny(ArrBadchars) == -1)
                {
                    sb.Append(inFilename.Substring(i, 1));
                }
            }
            return sb.ToString();
        }

        public static decimal SafeDecimalParseWithDefault(string inDecimal, decimal def)
        {
            try
            {
                if (string.IsNullOrEmpty(inDecimal)) return def;

                decimal outvalue = def;
                if (decimal.TryParse(GetNumericChars(inDecimal),
                                System.Globalization.NumberStyles.Any, null, out outvalue) == false)
                    return def;
                return outvalue;
            }
            catch
            {
                return def;
            }

        }

        /// <summary>
        /// returns just the numeric characters, plus the negative sign.
        /// </summary>
        /// <param name="inString"></param>
        /// <returns></returns>
        public static string GetNumericChars(string inString)
        {
            if (string.IsNullOrEmpty(inString))
                return "";

            StringBuilder sb = new StringBuilder(10);

            for (int i = 0; i < inString.Length; i++)
            {
                char c = inString[i];
                if (c == '.' || c == ',' || c == '-' || char.IsNumber(c))
                    sb.Append(c);
            }

            return sb.ToString();
        }
       
        public static DateTime SafeDateParse(string inDate)
        {
            try
            {
                if (string.IsNullOrEmpty(inDate)) return DateTime.MinValue;

                DateTime outDate = DateTime.MinValue;
                if (DateTime.TryParse(inDate, out outDate) == false)
                {
                    return DateTime.MinValue;
                }
                else
                    return outDate;
            }
            catch
            {
                return DateTime.MinValue;
            }
        }
        /// <summary>
        /// returns the date in shortdateString format, or blank if the date is <= 1900;
        /// </summary>
        /// <param name="inDate"></param>
        /// <returns></returns>
        public static string DateToShortDateString(DateTime inDate)
        {
            if (inDate <= Jan1900)
                return "";
            return inDate.Date.ToShortDateString();
        }
        /// <summary>
        /// Returns a SQL Server safe min date.
        /// </summary>
        public static DateTime Jan1900 = DateTime.Parse("1900/01/01 00:00:00");

        public static DateTime Jan2007 = DateTime.Parse("2007/01/01 00:00:00");

        public static DateTime Jan2008 = DateTime.Parse("2008/01/01 00:00:00");

        public static DateTime Jan2012 = DateTime.Parse("2012/01/01 00:00:00");

        public static DateTime Jan2030 = DateTime.Parse("2030/01/01 00:00:00");

        public static DateTime Jan2050 = DateTime.Parse("2050/01/01 00:00:00");

        public static DateTime
            SafeDateParseWithDefault(string inDate, DateTime inDefaultDate)
        {
            try
            {
                if (string.IsNullOrEmpty(inDate)) return inDefaultDate;
                DateTime outdate = DateTime.MinValue;
                if (DateTime.TryParse(inDate, out outdate) == false)
                {
                    return inDefaultDate;
                }
                return outdate;
            }
            catch
            {
                return inDefaultDate;
            }
        }
        public static DateTime SafeDateParseWithDefault(object inDate, DateTime inDefaultDate)
        {
            try
            {

                if (inDate == null) return inDefaultDate;

                if (inDate is DateTime)
                    return (DateTime)inDate;

                return SafeDateParseWithDefault(inDate.ToString(), inDefaultDate);
            }
            catch
            {
                return inDefaultDate;
            }
        }

       
        /// <summary>
        /// This method splits a string based on a delimiter, and returns a list of strings
        /// 
        /// Note: Blank strings are ignored
        /// </summary>
        /// <param name="inString"></param>
        /// <returns></returns>
        public static List<string> SafeSplitToList(string inString, char delimiter)
        {
            try
            {
                List<string> oList = new List<string>();
                foreach (string s in SafeSplit(inString, delimiter))
                {
                    if (string.IsNullOrEmpty(s)) continue;
                    oList.Add(s);
                }
                return oList;
            }
            catch
            {
                return new List<string>();
            }
        }
        /// <summary>
        /// This method splits a string based on a line feed.
        /// </summary>
        /// <param name="inString"></param>
        /// <returns></returns>
        public static string[] SafeSplit(string inString, char delimiter)
        {
            try
            {
                if (inString == null)
                    return "".Split('\n');
                return inString.Split(delimiter);
            }
            catch
            {
                return "".Split('\n');
            }
        }

        /// <summary>
        /// Splits a string based on a character, and trims the result strings
        /// </summary>
        /// <param name="inString"></param>
        /// <param name="delimiter"></param>
        /// <returns></returns>
        public static string[] SafeSplitAndTrim(string inString, char delimiter)
        {
            try
            {
                if (inString == null)
                    return "".Split('\n');
                List<string> oTemp = new List<string>();
                foreach (string s in inString.Split(delimiter))
                {
                    oTemp.Add(s.Trim());
                }
                return oTemp.ToArray();
            }
            catch
            {
                return "".Split('\n');
            }
        }

        /// <summary>
        /// This method splits a string based on a line feed.
        /// </summary>
        /// <param name="inString"></param>
        /// <returns></returns>
        public static string[] SafeSplit(string inString)
        {
            try
            {
                if (inString == null)
                    return "".Split('\n');

                // remove the \r characters so we don't double up on lines
                return inString.Replace("\r", "").Split('\n');
            }
            catch
            {
                return "".Split('\n');
            }
        }

        /// <summary>
        /// case-insensitive Contains extension
        /// </summary>
        /// <param name="s"></param>
        /// <param name="contains"></param>
        /// <returns></returns>
        public static bool Contains_IgnoreCase(this string s, string contains)
        {
            if (string.IsNullOrEmpty(s))
                return false;

            return s.IndexOf(contains, StringComparison.OrdinalIgnoreCase) >= 0;
        }
        
        
	}
}

